@@ -31,7 +31,7 @@ History:
 2010-03-11 ROwen    Changed the default for gridWdg to copy helpText and helpURL from the first data widget.
                     Bug fix: gridWdg would fail if helpText or helpURL was True and the first data widget
                     was missing the associated attribute.
-                
+2011-07-29 ROwen    _BaseGridSet: added __getitem__, __len__  and __length__ to simplify access to widgets.
 """
 __all__ = ['Gridder']
 
@@ -214,7 +214,7 @@ class Gridder(object):
         """
         self._defCol = int(col)
         self._nextCol = max(self._defCol, self._nextCol)
-        self._maxNextCol = max(self._nextCol, self._maxNextCol) 
+        self._maxNextCol = max(self._nextCol, self._maxNextCol)
     
     def setNextRow(self, row):
         """Sets the next row, the default row for the next call to gridWdg.
@@ -362,7 +362,7 @@ class _BaseGridSet:
                     columnspan = colSpan,
                 )
                 self.wdgSet.append(wdg)
-            if wdg != False:        
+            if wdg != False:
                 self.nextCol += colSpan
 
     def _makeWdg(self, wdgInfo):
@@ -407,6 +407,23 @@ class _BaseGridSet:
             self.helpText = getattr(firstWdg, "helpText", None)
         if self.helpURL == True:
             self.helpURL = getattr(firstWdg, "helpURL", None)
+    
+    def __getitem__(self, ind):
+        """Implement keyVar[ind] to return the specified value from the wdgSet.
+        
+        @raise IndexError if ind is out of range
+        """
+        return self.wdgSet[ind]
+
+    def __iter__(self):
+        """Implement for x in keyVar (appears to be redundant, but can't hurt)
+        """
+        return iter(self.wdgSet)
+
+    def __len__(self):
+        """Implement len(keyVar) to return the number of values
+        """
+        return len(self.wdgSet)
 
 
 class _GridSet (_BaseGridSet):
