@@ -1,16 +1,18 @@
 #!/usr/bin/env python
-"""A variant of Tkinter.OptionMenu that adds help, default handling,
-the ability to change menu items and the ability to configure the menu.
+"""A variant of Tkinter.OptionMenu that adds many features.
+
+Extra features include: help, default handling, the ability to change menu items
+and the ability to configure the menu.
+
 OptionMenu is essentially an RO.Wdg.Entry widget (though I don't promise
 that *all* methods are implemented).
 
 Note: I had to go mucking with internals, so some of this code is based on
 Tkinter's implementation of OptionMenu.
 
-To do:
-- Color menu items in autoIsCurrent mode.
-
 Warnings:
+- Do not change the width of the menubutton after creating the OptionMenu; that will conflict
+  with workarounds for Tcl/Tk bugs.
 - If "" is a valid option then be sure to set noneDisplay to something other than "".
   Otherwise getString will return the default value when the "" is selected.
 - As of Tk 8.4.19, MacOS X has poor visual support for background color (isCurrent)
@@ -90,6 +92,12 @@ History:
                     formerly it would return the default if the current value was noneDisplay,
                     which caused surprising behavior if noneDisplay was a valid value.
                     Added method isValid.
+2012-10-25 ROwen    If width is nonzero on aqua, increase it to work around Tk bug #3580194.
+2012-11-16 ROwen    If width is 0 on aqua, set it manually based on content to work around Tk bug #3587262.
+2012-11-29 ROwen    Fix demo and add demonstration of fixed width.
+2012-11-30 ROwen    Bug fix: width patch was not applied if width changed after the widget was created.
+                    Now it is applied by overridden method configure.
+2012-11-30 ROwen    Does no width correction if bitmap is shown.
 """
 __all__ = ['OptionMenu']
 
@@ -97,6 +105,7 @@ import Tkinter
 import RO.AddCallback
 import RO.Alg
 import RO.SeqUtil
+import RO.TkUtil
 from CtxMenu import CtxMenuMixin
 from IsCurrentMixin import AutoIsCurrentMixin, IsCurrentActiveMixin
 from SeverityMixin import SeverityActiveMixin
@@ -185,7 +194,7 @@ class OptionMenu (Tkinter.Menubutton, RO.AddCallback.TkVarMixin,
         if trackDefault == None:
             trackDefault = bool(autoIsCurrent)
         self.trackDefault = trackDefault
-
+        
         # handle keyword arguments for the Menubutton
         # start with defaults, update with user-specified values, if any
         # then set text or textvariable
@@ -205,7 +214,8 @@ class OptionMenu (Tkinter.Menubutton, RO.AddCallback.TkVarMixin,
         else:
             wdgKArgs["textvariable"] = var
         self.label = label
-        Tkinter.Menubutton.__init__(self, master, **wdgKArgs)
+        Tkinter.Menubutton.__init__(self, master)
+        self.configure(**wdgKArgs) # call overridden configure to fix width, if necessary
         self._menu = Tkinter.Menu(self, tearoff=False) # name="menu", tearoff=False)
         self["menu"] = self._menu
         # self.menuname = self._menu._w
@@ -222,30 +232,37 @@ class OptionMenu (Tkinter.Menubutton, RO.AddCallback.TkVarMixin,
         
         self.setItems(items, helpText=helpText, checkCurrent = False, checkDefault = False)
         self.setDefault(defValue, isCurrent = isCurrent, doCheck = True, showDefault = showDefault)
+        
+        # work around Tcl/Tk bug #3587262
+        if (RO.TkUtil.getWindowingSystem() == RO.TkUtil.WSysAqua) \
+            and RO.TkUtil.getTclVersion().startswith("8.5") \
+            and (kargs.get("width", 0) == 0):
+            self.addCallback(self._patchMacAutoWidth, callNow=True)
 
         # add callback function after setting default
         # to avoid having the callback called right away
         if callFunc:
-            self.addCallback(callFunc, False)
+            self.addCallback(callFunc, callNow=False)
+    
+    def configure(self, argDict=None, **kargs):
+        """Overridden version of configure that applies a width correction, if necessary
 
-    def _doCallbacks(self):
-        self._basicDoCallbacks(self)
-        if self._helpTextDict:
-            self.helpText = self._helpTextDict.get(self._var.get())
-
-    def _addItems(self):
-        """Adds the list of items to the menu;
-        must only be called when the menu is empty
-        and self._items has been set
+        Notes:
+        - configure is called by wdg[item] = value
+        - sometimes configure is called with a single positional argument: a dict of items,
+            and sometimes it is called with a set of keyword arguments. This code handles both cases.
+        - configure is NOT called by the widget's constructor, so you must call configure with your desired width
+            after constructing the widget, rather than passing width to the widget's constructor
         """
-        for item in self._items:
-            if item == None:
-                self._menu.add_separator()
-            else:
-                self._menu.add_command(
-                    label=item,
-                    command=_DoItem(self._var, item),
-                )
+        if argDict is not None:
+            kargs.update(argDict)
+        if "width" in kargs:
+            kargs["width"] = self._computeCorrectedWidth(
+                width = kargs["width"],
+                hasBitmap = bool(kargs.get("bitmap", self["bitmap"])),
+                showIndicator = kargs.get("indicatoron", self["indicatoron"]),
+            )
+        Tkinter.Menubutton.configure(self, **kargs)
     
     def asString(self, val):
         """Return display string associated with specified value:
@@ -529,6 +546,50 @@ class OptionMenu (Tkinter.Menubutton, RO.AddCallback.TkVarMixin,
         if self._helpTextDict:
             self.helpText = self._helpTextDict.get(self._var.get())
 
+    def _addItems(self):
+        """Adds the list of items to the menu;
+        must only be called when the menu is empty
+        and self._items has been set
+        """
+        for item in self._items:
+            if item == None:
+                self._menu.add_separator()
+            else:
+                self._menu.add_command(
+                    label=item,
+                    command=_DoItem(self._var, item),
+                )
+
+    def _doCallbacks(self):
+        self._basicDoCallbacks(self)
+        if self._helpTextDict:
+            self.helpText = self._helpTextDict.get(self._var.get())
+    
+    def _computeCorrectedWidth(self, width, hasBitmap, showIndicator):
+        """Compute corrected width to overcome Tcl/Tk bugs
+        """
+        if (width != 0) \
+            and not hasBitmap \
+            and (RO.TkUtil.getWindowingSystem() == RO.TkUtil.WSysAqua) \
+            and RO.TkUtil.getTclVersion().startswith("8.5"):
+            if showIndicator:
+                corrWidth = width + 3
+            else:
+                corrWidth = width + 2
+            return corrWidth
+        return width
+    
+    def _patchMacAutoWidth(self, wdg=None):
+        """Callback function that manually sets width to work around Tcl/Tk bug #3587262
+        
+        The effect of this bug is that the displayed width may be too narrow in auto mode (width=0)
+        on MacOS using Tcl/Tk 8.5. Thus you must only register this
+        
+        Only register this callback function on aqua Tcl/Tk 8.5
+        """
+        currVal = self._var.get()
+        self["width"] = len(currVal)
+
 
 if __name__ == "__main__":
     import Label
@@ -541,7 +602,7 @@ if __name__ == "__main__":
 
     items = ("Earlier", "Now", "Later", None, "Never")
     helpTexts = ("Help for Earlier", "Help for Now", "help for Later", "", "Help for Never")
-    menu = OptionMenu(root,
+    menu1 = OptionMenu(root,
         items = items,
         defValue = "Now",
         callFunc = callFunc,
@@ -549,12 +610,53 @@ if __name__ == "__main__":
         helpText = helpTexts,
         autoIsCurrent = True,
     )
-    menu.grid(row=0, column=0, sticky="w")
+    menu1.grid(row=0, column=0, sticky="w")
 
-    label = Label.Label(root, width=20, anchor="w", helpText="current value")
-    label.grid(row=1, column=0, sticky="w")
+    items = ("MmmmmNnnnn A", "MmmmmNnnnn B", "MmmmmNnnnn C")
+    menu2 = OptionMenu(root,
+        items = items,
+        defValue = "MmmmmNnnnn A",
+        callFunc = callFunc,
+        defMenu = "Default",
+        helpText = "width=0",
+    )
+    menu2.grid(row=0, column=1, sticky="w")
+
+    menu3 = OptionMenu(root,
+        items = items,
+        defValue = "MmmmmNnnnn A",
+        callFunc = callFunc,
+        defMenu = "Default",
+        helpText = "width=12 via configure",
+    )
+    menu3.configure(width=12)
+    menu3.grid(row=1, column=1, sticky="w")
+
+    menu4 = OptionMenu(root,
+        items = items,
+        defValue = "MmmmmNnnnn A",
+        callFunc = callFunc,
+        defMenu = "Default",
+        indicatoron = False,
+        helpText = "indicatoron=False",
+    )
+    menu4.grid(row=0, column=2, sticky="w")
+
+    menu5 = OptionMenu(root,
+        items = items,
+        defValue = "MmmmmNnnnn A",
+        callFunc = callFunc,
+        defMenu = "Default",
+        indicatoron = False,
+        helpText = "width=12 via ['width'], indicatoron=False",
+    )
+    menu5["width"] = 12
+    menu5.grid(row=1, column=2, sticky="w")
+
+    label = Label.Label(root, width=20, anchor="w", helpText="most recently selected value")
+    label.grid(row=2, column=0, columnspan=4, sticky="w")
     
     statusBar = StatusBar.StatusBar(root, width=20)
-    statusBar.grid(row=2, column=0, sticky="ew")
+    statusBar.grid(row=3, column=0, columnspan=4, sticky="ew")
 
     root.mainloop()
